// bme.c  
#include <stdio.h>
#include <string.h>
#include <stdint.h>
#include "pico/stdlib.h"
#include "math.h"
#include "hardware/i2c.h"
#include "hardware/adc.h"
#include "hardware/spi.h"
#include "hardware/uart.h"
#include "bme.h"
//=============================================
#define DISPLAY_SECS 5

void do_bme280()
{
   unsigned int xmit_count, display_count;

   uint16_t n;

// make sure uart FIFO is empty, disable FIFO for this
   uart_set_fifo_enabled(UART_ID, false);
   for(n=0; n<32; n++)   
   {
       // keep going until FIFO is empty (0 bytes in FIFO)
       if( uart_is_readable(UART_ID) )   { uart_char = uart_getc(UART_ID); }
       else {break;}
   }
   //uart_set_fifo_enabled(UART_ID, true);  // Keep it off
//-------------------------------
   bme_init();   // Initialize bme

   strcpy(line,"Device ID: ");  // Get device ID
   itoa_hex(device_id);
   strcat(line,nstr);
   if(device_id == 0x60)  { strcat(line, "  BME-280\n\n"); }
   if(device_id == 0x58)  { strcat(line, "  BMP-280\n\n"); }
   if( (device_id != 0x60) && (device_id != 0x58) )  { strcat(line, "unknown device\n\n"); }
   uart_puts(UART_ID, line);   // send it out

   sleep_ms(3000);
   
   getcal();     // Do calibration data for temperature and humidity and make digs

   log_cal();
   log_digs();

   read_bme280();
   compensate_temperature();  // global Fdeg = temperature in Fahrenheit
   log_vars();

   compensate_pressure();     // global p = pressure in mb
   compensate_humidity();     // global Humidity in %

   show_temperature = Fdeg;   // for testing the sensor  7403 is 2430 is 24.3 degrees C is 75.7 degrees F
   show_humidity = Humidity;  // for testing the sensor  23 is 23%
   show_pressure = Pressure;         // for testing the sensor  "96386" equals 96386 Pa = 963.86 hPa

   asm(" nop");               // for debugger, break here to check that the sensor is working

   log_raw();
   log_wx_data_ts();    // send data out through uart for logging the wx data
   show_wx_data_lcd();

   sleep_ms(2000);

   uart_puts(UART_ID, "Keyboard Commands (Upper or Lower Case)\n");
   uart_puts(UART_ID, "   V: Show Version\n");
   uart_puts(UART_ID, "   B: Show BME-280\n");
   uart_puts(UART_ID, "   O: Log BME-280 Once\n");
   uart_puts(UART_ID, "   L: Log BME-280 Every 5 minutes\n");
   uart_puts(UART_ID, "   X: Stop Logging BME-280\n");
   uart_puts(UART_ID, "   D: Dump All of the Flash Logs to uart\n");
   uart_puts(UART_ID, "   E: Erase Data Flash?  (then Y/N)\n"); 
   uart_puts(UART_ID, "   S: Reset Start Date?  (then Y/N)\n");
   uart_puts(UART_ID, "   H: Repeat This List\n\n"); 

   uart_puts(UART_ID, "> Ready For Commands\n\n"); 
//-----------------------
// Main Loop 
   display_count = DISPLAY_SECS;

   while(1)
   {
      if(secs_flag)
      {

         read_bme280();
         compensate_temperature();     // global Fdeg = temperature in Fahrenheit
         compensate_pressure();        // global p = pressure in mb
         compensate_humidity();        // global Humidity in %

         display_count--;
         xmit_count--;

         if(display_count == 0)    // for lcd
            { display_count = DISPLAY_SECS; }

         if(display_count == 2)  
         { 
            gpio_put(LED1,1);
            show_wx_data_lcd();     // show wx data on lcd
         }
         else { gpio_put(LED1,0); }

         if( --bme_log_count == 0 )    // for logging bme in Flash
         { 
            if( bme_log_flag )    // log it in flash
            {
               read_bme280();
               compensate_temperature();     // global Fdeg = temperature in Fahrenheit
               compensate_pressure();        // global p = pressure in mb
               compensate_humidity();        // global Humidity in %

               make_wx_entry(&flash_entry); // make a log entry from Fdeg, p, Humidity 
               for(n = 0; n<256; n++)  { temp_buffer[n] = 0xFF; }
               memcpy(temp_buffer,&flash_entry, 32);
               write_page(Page_Index, temp_buffer);
               show_log_page(Page_Index);
               Page_Index++;
            }
            
            bme_log_count = 300;  // 5 minutes
         }
         
         secs_flag = 0;
      
      } // end secs

     //--------------------------------    
     do_uart_RX();    // Use uart RX to decide what to send to uart
     //--------------------------------

   }  // end while

}  // end function
//==============================================
// Use uart RX to decide what to send to uart
void do_uart_RX()
{
   uint16_t n;
   uint32_t page_end;

   if( uart_is_readable(UART_ID) )   
   { 
      uart_char = uart_getc(UART_ID) & (~0x20);    // read uart FIFO (upper or lower case)

      switch(uart_char)
      {
         case 'O':   // Log ADC Once
            read_bme280();
            compensate_temperature();     // global Fdeg = temperature in Fahrenheit
            compensate_pressure();        // global p = pressure in mb
            compensate_humidity();        // global Humidity in %

            make_wx_entry(&flash_entry); // make a log entry from Fdeg, p, Humidity 
            for(n = 0; n<256; n++)  { temp_buffer[n] = 0xFF; }
            memcpy(temp_buffer,&flash_entry, 32);
            write_page(Page_Index, temp_buffer);
            show_log_page(Page_Index);
            Page_Index++;
            break;
         //----------
         case 'V':   // Version message
            strcpy(line, version);
            strcat(line,"      Start Date: ");
            strcat(line, start_date);
            uart_puts(UART_ID, line); 
            break;
         //----------
         case 'B':   // Show BME-280
            read_bme280();
            compensate_temperature();     // global Fdeg = temperature in Fahrenheit
            compensate_pressure();        // global p = pressure in mb
            compensate_humidity();        // global Humidity in %

            make_wx_entry(&flash_entry); // make a log entry from Fdeg, p, Humidity 
            show_entry(&flash_entry);
            break;
         //----------
          case 'L':  // Log ADC Every 5 minutes
            bme_log_count = 10;     // for a fast start
            bme_log_flag = 1;
            ts_number = 0;        // time stamp number
            uart_puts(UART_ID, "Logging ADC every 5 minutes. Press X to stop\n\n"); 
            break;
         //----------
          case 'X':  // Stop Logging BME
            bme_log_flag = 0;
            uart_puts(UART_ID, "ADC Logging Stopped\n\n"); 
            break;
         //----------
          case 'D':    // Dump All of the Flash logs to uart
            uart_puts(UART_ID,"\nFull Flash Log\n");      
            page_end = Page_Index - 1;
            erase_flag = 0;
            dump_flash(0, page_end);
            break;           
          //----------
         case 'E':    // Erase Flash Data
            uart_puts(UART_ID, "\nErase Data Flash?   Y/N\n"); 
            erase_flag = 1;
            break;

         case 'S':    // Reset Start Date
            uart_puts(UART_ID, "\nReset Start Date?  Y/N\n\n");
            reset_rx_start = 1;
            break;

         case 'Y':    // Yes
            if(erase_flag)  
            {
               erase_all_sectors();
               uart_puts(UART_ID, "\nFlash Data Erased\n\n"); 
            }
            erase_flag = 0;

            if(reset_rx_start)  
            {
               do_start_date(1);  // Erase Start Date in Flash. write today's date
            }
            reset_rx_start = 0;
            break;

         case 'N':    // No
            erase_flag = 0;
            reset_rx_start = 0;
            uart_puts(UART_ID, "\nFlash Data Not Erased\n\n"); 
            break;
         //----------
         case 'H':    // Repeat this List
            uart_puts(UART_ID, "Keyboard Commands (Upper or Lower Case)\n");
            uart_puts(UART_ID, "   V: Show Version\n");
            uart_puts(UART_ID, "   B: Show BME-280\n");
            uart_puts(UART_ID, "   O: Log BME-280 Once\n");
            uart_puts(UART_ID, "   L: Log BME-280 Every 5 minutes\n");
            uart_puts(UART_ID, "   X: Stop Logging BME-280\n");
            uart_puts(UART_ID, "   D: Dump All of the Flash Logs to uart\n");
            uart_puts(UART_ID, "   E: Erase Data Flash?  (then Y/N)\n"); 
            uart_puts(UART_ID, "   S: Reset Start Date?  (then Y/N)\n");
            uart_puts(UART_ID, "   H: Repeat This List\n\n"); 
            break;
         //----------
         default:
            erase_flag = 0;
 
      }  // end switch
   }  // end of if uart was readable
} // end uart RX
//====================================================
// Read the BME280 data for pressure, temperature, and humidity
void read_bme280(void)
{

// Get the raw bme280 data
/*  bmedata
press_msb 0xF7 bmedata[0]       press_lsb 0xF8 bmedata[1]      press_xlsb 0xF9  bmedata[2]
       msb                              lsb                            xlsb  0000
       7-0                              7-0                            7-4   0000 
      23-16                            15-8                            7-4   3-0

temp_msb  0xFA  bmedata[3]     temp_lsb  0xFB  bmedata[4]      tmp_xlsb 0xFC bmedata[5]
       msb                              lsb                            xlsb  0000
       7-0                              7-0                            7-4   0000 
      23-16                            15-8                            7-4    3-0

hum_msb 0xFD bmedata[6]      hum_lsb 0xFE bmedata[7] 
         15-8                         7 - 0
*/

   unsigned char address;

   address = bme_ctrl_meas;
   i2c_write_byte_bme(address,0x2D);  // In forced mode it must be called for every reading
   sleep_ms(5);

   i2c_outbuffer[0] = press_msb;  // register address
   i2c_write_blocking(i2c_default, BME_ID, i2c_outbuffer, 1, true);
   i2c_read_blocking(i2c_default, BME_ID, bmedata, 8, false);

// Pressure - It is done this way so shifting a signed number doesn't screw it up
   adc_p = bmedata[0];
   adc_p = adc_p<<8;
   adc_p = adc_p | bmedata[1];
   adc_p = adc_p<<8;
   adc_p = adc_p | bmedata[2];
   adc_p = adc_p>>4;

// Temperature
   adc_t = bmedata[3];
   adc_t = adc_t<<8;
   adc_t = adc_t | bmedata[4];
   adc_t = adc_t<<8;
   adc_t = adc_t | bmedata[5];
   adc_t = adc_t>>4;

// Humidity
   adc_h = bmedata[6]<<8 | bmedata[7]; 
   asm(" nop");    // for debugger
}
//==========================================================
// Compensation for Temperature
int do_compensation()
{
   int32_t var1, var2, t_fine;
   uint16_t sign;
   int32_t lbin;

//   log_raw();
//   return 1;

// Temperature Compensation
   var1 = ( ( ( (adc_t>>3) - ( (int32_t)dig_T1<<1) ) ) * ( (int32_t)dig_T2) )>>11;
testvars[0] = var1;

   var2 = (((((adc_t>>4) - ((int32_t)dig_T1)) * ((adc_t>>4) - ((int32_t)dig_T1)))>>12) * ((int32_t)dig_T3))>>14;
testvars[1] = var2;

    t_fine = var1 + var2;
testvars[2] = t_fine;

    T = (t_fine * 5 + 128)>>8;   // Example: T = 2508 = 25.08 deg C
testvars[3] = T;

// Scale and convert to Fahrenheit
   if(T<0)  {sign = 1;}
   else     {sign = 0;}

   T = T/10;   // Example: 2508 -> 250 

  if(sign)  { lbin = (T * 18) - 3200;}
  else { lbin = (T * 18) + 3200;}    // Example:  2508 -> 7714

   Fdeg = lbin/10;  // Example: 2508 -> 771 = 0x0302
   temperature = Fdeg;    // signed int
testvars[4] = Fdeg;
//-------------------------------
   log_vars();

    return 1;
}
//===================================================
void bme_init()
{
    unsigned char address;

// BME280 reset
    address = bme_reset;
    i2c_write_byte_bme(address,0x00);
    sleep_us(100);

// Initialize the BME-280
// config    d7   d6   d5 | d4   d3   d2 | d1 | d0
//              t_sb          filter        x   spi3W_en
//            0    0    0    0    0    0    0    0
// t_sb = 000 (0.5 ms  standby time
// filter = 000  filter off
// spi3W_en = 0 for 4-wire
    address = bme_config;
    i2c_write_byte_bme(address,0x00);
    sleep_us(100);

// humidity: oversampling x1   ('0' = sensor off)
    i2c_write_byte_bme(0xF2,0x01);
    sleep_us(100);

// ctrl_meas    d7   d6   d5 |  d4   d3   d2 | d1   d0
//                 osrs_t           osrs_p       mode
//               0    0    1     0    1    1   0     1
// osrs_t = 001  oversampling x 1 17-bit  ('0' = sensor off)
// osrs_p = 011 oversampling x4   ('0' = off)
// mode = 11 (normal)  (0 = sleep; 01 and 10 = Forced Mode)
   address = bme_ctrl_meas;
   i2c_write_byte_bme(address,0x2D);  // In forced mode it must be called for every reading
   sleep_us(100);

   wait_ready();
}
//=================================================
// Read one i2C byte from bme
unsigned char i2c_read_byte(unsigned char adr)
{
   i2c_outbuffer[0] = adr;  // register address
   i2c_write_blocking(i2c_default, BME_ID, i2c_outbuffer, 1, true);
   i2c_read_blocking(i2c_default, BME_ID, i2c_inbuffer, 1, false);
   return(i2c_inbuffer[0]);
}
//==================================
// Read i2C bytes from bme
void i2c_read(unsigned char adr, unsigned int len)
{
   i2c_outbuffer[0] = adr;  // register address
   i2c_write_blocking(i2c_default, BME_ID, i2c_outbuffer, 1, true);
   i2c_read_blocking(i2c_default, BME_ID, i2c_inbuffer, len, false);
}
//=======================================================
// Write one i2C byte to bme
void i2c_write_byte_bme(unsigned char adr, unsigned char data)
{
   i2c_outbuffer[0] = adr;  // register address
   i2c_outbuffer[1] = data;  // register data

   i2c_write_blocking(i2c_default, BME_ID, i2c_outbuffer, 2, false);
}
//==================================
int wait_ready()
{
    int n, timeout;
    unsigned char busy, data;

    busy = 1;
    n = 1000;
    timeout = 0;

    while(busy)
    {
       data = i2c_read_byte(0xF3);
       if( !(data & 0x08)) {busy = 0;}
       if(n == 0)  {timeout = 1;}
       n--;
    }
    return timeout;
}
//==================================
// Get the calibration values
void getcal()
{
// Temperature
// Make digs for compensation functions. All digs are global
// Requires that the calata array be unsigned

// caldata[0] = dig_T1 lsb; caldata[1] = dig_T1 msb unsigned
   dig_T1 = caldata[1]<<8 | caldata[0];

// caldata[2] = dig_T2 lsb; caldata[3] = dig_T2 msb signed
   dig_T2 = caldata[3]<<8 | caldata[2];

// caldata[4] = dig_T3 lsb; caldata[5] = dig_T3 msb signed
   dig_T3 = caldata[5]<<8 | caldata[4];

// Pressure
// caldata[6] = dig_P1 lsb; caldata[7] = dig_P1 msb unsigned
   dig_P1 = caldata[7]<<8 | caldata[6];

// caldata[8] = dig_P2 lsb; caldata[9] = dig_P2 msb signed
   dig_P2 = caldata[9]<<8 | caldata[8];
//   if(dig_P2 & 0x10000) { dig_P2 |= 0xFFFF0000;}  // make signed

// caldata[10] = dig_P3 lsb; caldata[11] = dig_P3 msb    signed
   dig_P3 = caldata[11]<<8 | caldata[10];

// caldata[12] = dig_P4 lsb; caldata[13] = dig_P4 msb    signed
   dig_P4 = caldata[13]<<8 | caldata[12];

// caldata[14] = dig_P5 lsb; caldata[15] = dig_P5 msb    signed
   dig_P5 = caldata[15]<<8 | caldata[14];

// caldata[16] = dig_P6 lsb; caldata[17] = dig_P6 msb    signed
   dig_P6 = caldata[17]<<8 | caldata[16];

// caldata[18] = dig_P7 lsb; caldata[19] = dig_P7 msb    signed
   dig_P7 = caldata[19]<<8 | caldata[18];

// caldata[20] = dig_P8 lsb; caldata[21] = dig_P8 msb    signed
   dig_P8 = caldata[21]<<8 | caldata[20];

// caldata[22] = dig_P9 lsb; caldata[23] = dig_P9 msb    signed
   dig_P9 = caldata[23]<<8 | caldata[22];

   wait_ready();

// make the humidity digs
   // humdata[0] is dig_H3 7:0 (unsigned char);
   dig_H1 = humdata[0];  //  7:0

   // humdata[1] is dig_H2 7:0; humdata[2] is dig_H2 8:15
   dig_H2 = (humdata[2]<<8) | humdata[1];

   // humdata[3] is dig_H3 7:0  (unsigned char);
   dig_H3 = humdata[3];   // dig_H3 7:0

   // humdata[4] (7:0) is dig_H4 11"4; humdata[5] (3:0) is dig_H4 3:0
   dig_H4 = (humdata[4] <<4) | (humdata[5] & 0x0F);

   // humdata[5] 7:4 is also dig_H5 3:0; humdata[6] is dig_H5 11:4
   dig_H5 = ( (humdata[5]>>4) & 0x0F) | (humdata[6]<<4);

   dig_H6 = humdata[7];     // signed char
}
//=========================================
// Convert Binary signed temperature to BCD; 4 digits
// temperature from bme280 is in Celsius 
void temperature_itoa(char *str)      // remove leading zeros; end with NULL
{
   unsigned char bcd0, bcd1, bcd2, bcd3;
   unsigned char char1, char2, char3, char4;

   int i;
   unsigned char hcarry;
   unsigned int bin, sign;

// 0x08DD = 2269.; 2269/10 = 226; 226 * 18 = 4084; 4084 + 3200 = 7284 = 72.8 deg F;  

// 0xF723 = -2269.; -2269/10 = -226; 226 * 18 = -4084; -4084 + 3200 = -884 = -8.8 deg F;  

   if(Fdeg < 0)
   {
      bin = -Fdeg;
      sign = 1; 
   }
   else
   { 
       bin = Fdeg;
       sign = 0;
   }

// Convert to BCD
   bcd0 = 0;
   bcd1 = 0;
   bcd2 = 0;
   bcd3 = 0;

   for(i=0; i<16; i++)
   {
//---------- BCD0
      bcd0 = bcd0<<1;

      if(bin & 0x8000)
      {bcd0 = bcd0 +1;}

      bin = bin<<1;

      if(bcd0 >9)
      {
         bcd0 = bcd0 - 10;
         hcarry = 1;
      }
     else {hcarry = 0; }

//---------- BCD1
      bcd1 = bcd1<<1;
      bcd1 = bcd1 + hcarry;

      if(bcd1 >9)
      {
         bcd1 = bcd1 - 10;
         hcarry = 1;
      }
      else {hcarry = 0;}

//---------- BCD2
      bcd2 = bcd2<<1;
      bcd2 = bcd2 + hcarry;

      if(bcd2 >9)
      {
         bcd2 = bcd2 - 10;
         hcarry = 1;
      }
      else {hcarry = 0;}

//---------- BCD3
      bcd3 = bcd3<<1;
      bcd3 = bcd3 + hcarry;

      if(bcd3 >9)
      {
          bcd3 = bcd3 - 10;
//          hcarry = 1;
      }
//      else {hcarry = 0;}

   }  // end of converting to bcd

// Convert to ascii
   char1 = bcd3 + 0x30;
   char2 = bcd2 + 0x30;
   char3 = bcd1 + 0x30;
   char4 = bcd0 + 0x30;

/*
// Make the string; do not remove leading zeros
   if(sign)  
   { 
      str[0] = '-';
      str[1] = char1;
      str[2] = char2;
      str[3] = char3;
      str[4] = '.';
      str[5] = char4;
      str[6] = 0;
   }
   else
   {
      str[0] = char1;
      str[1] = char2;
      str[2] = char3;
      str[3] = '.';
      str[4] = char4;
      str[5] = 0;
   }
*/

// Make the string; remove leading zeros
    if(sign)    // number is negative
    { 
       str[0] = '-';

       if(char1 != 0x30)
       { 
          str[1] = char1;
          str[2] = char2;
          str[3] = char3;
          str[4] = '.';
          str[5] = char4;
          str[6] = 0;
          for(i=0; i<7; i++)  {temperature_lcd[i] = str[i];}  // for displaying on lcd later
          return;
       }
     
       if(char2 != 0x30)
       { 
          str[1] = char2;
          str[2] = char3;
          str[3] = '.';
          str[4] = char4;
          str[5] = 0;
          for(i=0; i<6; i++)  {temperature_lcd[i] = str[i];}  // for displaying on lcd later
          return;
       }  

      // display at least two characters
      str[1] = char3;
      str[2] = '.';
      str[3] = char4;
      str[4] = 0;
      for(i=0; i<5; i++)  {temperature_lcd[i] = str[i];}   // for displaying on lcd later
    }  // end of sign is negative
    else   // sign is positive
    {
       if(char1 != 0x30)
       { 
          str[0] = char1;
          str[1] = char2;
          str[2] = char3;
          str[3] = '.';
          str[4] = char4;
          str[5] = 0;
          for(i=0; i<6; i++)  {temperature_lcd[i] = str[i];}  // for displaying on lcd later
          return;
       }
     
       if(char2 != 0x30)
       { 
          str[0] = char2;
          str[1] = char3;
          str[2] = '.';
          str[3] = char4;
          str[4] = 0;
          for(i=0; i<5; i++)  {temperature_lcd[i] = str[i];}  // for displaying on lcd later
          return;
       }  

      // display at least two characters
      str[0] = char3;
      str[1] = '.';
      str[2] = char4;
      str[3] = 0;
      for(i=0; i<4; i++)  {temperature_lcd[i] = str[i];}    // for displaying on lcd later
    }  // end of sign is positive

}
//==========================================================
void show_wx_data_lcd()
{
   int n;

   temperature_itoa(nstr);   // remove leading zeros; end with NULL
   pressure_ltoa(nstr);      // remove leading zeros; end with NULL
   humidity_btoa(nstr);

   SetCmdMode();      // set for commands
   lcd_byte(0x01);    // clear screen

// Write Temperature
// T:
// Temperature
   lcd_wr_instr(0x80);  // position
   lcd_wr_data('T');
   lcd_wr_data(':');
   lcd_wr_data(' ');
   for(n=0; n<10; n++)
   {
      if(temperature_lcd[n] == 0)  {break;}   // NULL character
      lcd_wr_data(temperature_lcd[n]);
   }
   lcd_wr_data(' ');
   lcd_wr_data('F');

// H:
// Humidity
   lcd_wr_instr(0x8B);  // position
   lcd_wr_data('H');
   lcd_wr_data(':');
//   lcd_wr_data(' ');
   for(n=0; n<2; n++)
   {
      if(humidity_lcd[n] == 0)  {break;}   // NULL character
      lcd_wr_data(humidity_lcd[n]);
   }
//   lcd_wr_data(' ');
   lcd_wr_data('%');

// P:
// Pressure
   lcd_wr_instr(0xC0);  // position
   lcd_wr_data('P');
   lcd_wr_data(':');
   lcd_wr_data(' ');
   for(n=0; n<10; n++)
   {
      if(pressure_lcd[n] == 0)  {break;}   // NULL character
      lcd_wr_data(pressure_lcd[n]);
   }
   lcd_wr_data(' ');
   lcd_wr_data('m');
   lcd_wr_data('b');
}
//====================================================
void log_wx_data_ts()
{
   log_time();   // time stamp in line

   strcat(line,"Temperature: ");
   temperature_itoa(nstr);      // remove leading zeros; end with NULL
   strcat(line,nstr);
   strcat(line," Degrees    ");

   strcat(line,"Pressure: ");
   pressure_ltoa(nstr);        // remove leading zeros; end with NULL
   strcat(line,nstr);
   strcat(line," mb    ");

   strcat(line,"Humidity: ");
   humidity_btoa(nstr);
   strcat(line,nstr);

   strcat(line," %\r\n\n");

   uart_puts(UART_ID,line);  
}
//===========================================================
void log_wx_devid_ts()
{
   strcpy(line,"Device ID: ");
   devid(nstr);
   strcat(line,nstr);
   strcat(line,"\r\n\n");

   uart_puts(UART_ID,line);  
}
//===========================================================
// Convert hex pressure to BCD; 6 digits
void pressure_ltoa(char *str)      // remove leading zeros; end with NULL
{
   unsigned char bcd0, bcd1, bcd2, bcd3, bcd4, bcd5;
   unsigned char char1, char2, char3, char4, char5;

   int i;
   unsigned char hcarry;
   unsigned long bin;

   bin = Pressure;

// Adapted from 8-bit microcontroller code using a DAA instruction
   bcd0 = 0;
   bcd1 = 0;
   bcd2 = 0;
   bcd3 = 0;
   bcd4 = 0;
   bcd5 = 0;

   for(i=0; i<24; i++)
   {
//---------- BCD0
      bcd0 = bcd0<<1;

      if(bin & 0x800000)
      {bcd0 = bcd0 +1;}

      bin = bin<<1;

      if(bcd0 >9)
      {
         bcd0 = bcd0 - 10;
         hcarry = 1;
      }
     else {hcarry = 0; }

//---------- BCD1
      bcd1 = bcd1<<1;
      bcd1 = bcd1 + hcarry;

      if(bcd1 >9)
      {
         bcd1 = bcd1 - 10;
         hcarry = 1;
      }
      else {hcarry = 0;}

//---------- BCD2
      bcd2 = bcd2<<1;
      bcd2 = bcd2 + hcarry;

      if(bcd2 >9)
      {
         bcd2 = bcd2 - 10;
         hcarry = 1;
      }
      else {hcarry = 0;}

//---------- BCD3
      bcd3 = bcd3<<1;
      bcd3 = bcd3 + hcarry;

      if(bcd3 >9)
      {
          bcd3 = bcd3 - 10;
          hcarry = 1;
      }
      else {hcarry = 0;}

//---------- BCD4
      bcd4 = bcd4<<1;
      bcd4 = bcd4 + hcarry;

      if(bcd4 >9)
      {
         bcd4 = bcd4 - 10;
         hcarry = 1;
      }
      else {hcarry = 0;}

//---------- BCD5
      bcd5 = bcd5<<1;
      bcd5 = bcd5 + hcarry;

      if(bcd5 >9)
      {
         bcd5 = bcd5 - 10;
//         hcarry = 1;
      }
//      else {hcarry = 0;}

   }  // end of converting to bcd

// Convert to ascii
   char1 = bcd5 + 0x30;
   char2 = bcd4 + 0x30;
   char3 = bcd3 + 0x30;
   char4 = bcd2 + 0x30;
   char5 = bcd1 + 0x30;

/*
   str[0] = char1;
   str[1] = char2;
   str[2] = char3;
   str[3] = char4;
   str[4] = '.';
   str[5] = char5;
   str[6] = 0;
*/

// Make the string; remove leading zeros
   if(char1 != 0x30)
   { 
      str[0] = char1;
      str[1] = char2;
      str[2] = char3;
      str[3] = char4;
      str[4] = '.';
      str[5] = char5;
      str[6] = 0;      // NULL
      for(i=0; i<7; i++)  {pressure_lcd[i] = str[i];}  // for displaying on lcd
      return;
   }
   
   if(char2 != 0x30)
   { 
      str[0] = char2;
      str[1] = char3;
      str[2] = char4;
      str[3] = '.';
      str[4] = char5;
      str[5] = 0;    // NULL
      for(i=0; i<6; i++)  {pressure_lcd[i] = str[i];}  // for displaying on lcd
      return;
   }

   if(char3 != 0x30)
   { 
      str[0] = char3;
      str[1] = char4;
      str[2] = '.';
      str[3] = char5;
      str[4] = 0;     // NULL
      for(i=0; i<5; i++)  {pressure_lcd[i] = str[i];}  // for displaying on lcd
      return;
   }

   str[0] = char4;
   str[1] = '.';
   str[2] = char5;
   str[3] = 0;     // NULL
   for(i=0; i<4; i++)  {pressure_lcd[i] = str[i];}  // for displaying on lcd

   asm(" nop");    // for emulator
}
//=============================================
// Convert Binary signed humidity to BCD - 4 digits
void humidity_btoa(char *str)      // remove leading zeros; end with NULL
{
   unsigned char bcd0, bcd1;
   unsigned char char1, char2;

   int i;
   unsigned char hcarry;
   unsigned char bin;

   bin = Humidity;

   if(bin>99)  bin=99;   // maximum number

// Adapted from 8-bit microcontroller code using a DAA instruction
   bcd0 = 0;
   bcd1 = 0;

   for(i=0; i<8; i++)
   {
//---------- BCD0
      bcd0 = bcd0<<1;

      if(bin & 0x80)
      {bcd0 = bcd0 +1;}

      bin = bin<<1;

      if(bcd0 >9)
      {
         bcd0 = bcd0 - 10;
         hcarry = 1;
      }
     else {hcarry = 0; }

//---------- BCD1
      bcd1 = bcd1<<1;
      bcd1 = bcd1 + hcarry;

      if(bcd1 >9)
      {
         bcd1 = bcd1 - 10;
//         hcarry = 1;
      }
       else {hcarry = 0;}
   }  // end of converting to bcd

// Convert to ascii
   char1 = bcd1 + 0x30;
   char2 = bcd0 + 0x30;

// Make the string; remove leading zeros
   if(char1 != 0x30)
   { 
      str[0] = char1;
      str[1] = char2;
      str[2] = 0;
      for(i=0; i<3; i++)  {humidity_lcd[i] = str[i];}
   }
   else
   {
      str[0] = char2;
      str[1] = 0;
      for(i=0; i<2; i++)  {humidity_lcd[i] = str[i];}  // for displaying on lcd
   }  

}
//==================================================
void itoa_hex(unsigned char bin)
{
   unsigned char1, char2, temp;

// Convert to ascii
   temp = bin & 0x0F;
   if(temp<10)
     {char2 = temp + 0x30;}
   else
     {char2 = temp + 0x37;}

   temp = bin>>4 & 0x0F;
   if(temp<10)
     {char1 = temp + 0x30;}
   else
     {char1 = temp + 0x37;}
    
// Add to string
   nstr[0] = char1;
   nstr[1] = char2;
   nstr[2] = 0;  // NULL
}
//=========================================
void itoa_hex_int(unsigned int bin)
{
   unsigned char cdigit[8], m, n;
   unsigned int temp;

//   bin = 0x1234;  // for testing

   m = 3;

// Convert to ascii
   for(n=0; n<4; n++)
   {
     temp = bin & 0x0F;
     if(temp<10)  {cdigit[n] = temp + 0x30;}
     else {cdigit[n] = temp + 0x37;}
     bin = bin>>4;
     nstr[m--] = cdigit[n];
   }

   nstr[n] = 0;   // NULL
}
//=========================================
void itoa_hex_long(unsigned long bin)
{
   unsigned char cdigit[8], m, n;
   unsigned long temp;

//   bin = 0x12345678;   // for testing

   m = 7;
// Convert to ascii
   for(n=0; n<8; n++)
   {
     temp = bin & 0x0F;
     if(temp<10)  {cdigit[n] = temp + 0x30;}
     else {cdigit[n] = temp + 0x37;}
     bin = bin>>4;
     nstr[m--] = cdigit[n];
   }

   nstr[n] = 0;   // NULL
}
//=========================================
void log_cal()
{
   unsigned char k;

   strcpy(line,"Cal Data 0x88 to 0x8F (24 bytes)\r\n");

   for(k = 0; k<24; k++)
   {
      itoa_hex(caldata[k]);
      strcat(line,nstr);
      strcat(line,"   ");
   }

   strcat(line,"\nCal Hum (8 bytes)   ");
 
   for(k = 0; k<8; k++)
   {
      itoa_hex(humdata[k]);
      strcat(line,nstr);
      strcat(line,"   ");
   }

   strcat(line,"\r\n\n");
   uart_puts(UART_ID,line);  
}

/*
unsigned short dig_T1, dig_P1;
long int dig_T2, dig_T3, dig_P2, dig_P3, dig_P4, dig_P5, dig_P6, dig_P7, dig_P8, dig_P9;
unsigned char dig_H1, dig_H3;
int dig_H2, dig_H4, dig_H5;
char dig_H6;
*/
//=======================================
void log_digs()
{
   strcpy(line,"dig T1: ");
   itoa_hex_int(dig_T1);
   strcat(line,nstr);

   strcat(line,"   dig T2: ");
   itoa_hex_int(dig_T2);
   strcat(line,nstr);

   strcat(line,"   dig T3: ");
   itoa_hex_int(dig_T3);
   strcat(line,nstr);

   strcat(line,"\n\ndig P1: ");
   itoa_hex_int(dig_P1);
   strcat(line,nstr);

   strcat(line,"    dig P2: ");
   itoa_hex_int(dig_P2);
   strcat(line,nstr);

   strcat(line,"    dig P3: ");
   itoa_hex_int(dig_P3);
   strcat(line,nstr);

   strcat(line,"    dig P4: ");
   itoa_hex_int(dig_P4);
   strcat(line,nstr);

   strcat(line,"\ndig P5: ");
   itoa_hex_int(dig_P5);
   strcat(line,nstr);

   strcat(line,"    dig P6: ");
   itoa_hex_int(dig_P6);
   strcat(line,nstr);

   strcat(line,"    dig P7: ");
   itoa_hex_int(dig_P7);
   strcat(line,nstr);

   strcat(line,"    dig P8: ");
   itoa_hex_int(dig_P8);
   strcat(line,nstr);

   strcat(line,"    dig P9: ");
   itoa_hex_int(dig_P9);
   strcat(line,nstr);

   strcat(line,"\n\ndig H1: ");
   itoa_hex(dig_H1);
   strcat(line,nstr);

   strcat(line,"    dig H2: ");
   itoa_hex_int(dig_H2);
   strcat(line,nstr);

   strcat(line,"    dig H3: ");
   itoa_hex(dig_H3);
   strcat(line,nstr);

   strcat(line,"    dig H4: ");
   itoa_hex_int(dig_H4);
   strcat(line,nstr);

   strcat(line,"    dig H5: ");
   itoa_hex_int(dig_H5);
   strcat(line,nstr);

   strcat(line,"    dig H6: ");
   itoa_hex(dig_H6);
   strcat(line,nstr);

   strcat(line,"\r\n\n");
   uart_puts(UART_ID,line);  
}
//===============================================
void log_raw()
{
   strcpy(line,"raw data:\n");

   strcat(line,"pres_msb: ");
   itoa_hex(bmedata[0]);
   strcat(line,nstr);

   strcat(line,"    pres_lsb: ");
   itoa_hex(bmedata[1]);
   strcat(line,nstr);

   strcat(line,"    pres_xlsb: ");
   itoa_hex(bmedata[2]);
   strcat(line,nstr);

   strcat(line,"\ntemp_msb: ");
   itoa_hex(bmedata[3]);
   strcat(line,nstr);

   strcat(line,"   temp_lsb: ");
   itoa_hex(bmedata[4]);
   strcat(line,nstr);

   strcat(line,"   temp_xlsb: ");
   itoa_hex(bmedata[5]);
   strcat(line,nstr);

   strcat(line,"\nhum_msb: ");
   itoa_hex(bmedata[6]);
   strcat(line,nstr);

   strcat(line,"   hum_lsb:");
   itoa_hex(bmedata[7]);
   strcat(line,nstr);

   strcat(line,"\r\n\n");
   uart_puts(UART_ID,line);  

   strcpy(line,"adc_p: ");      // unsigned
   itoa_hex_long(adc_p);
   strcat(line,nstr);

   strcat(line,"    adc_t: ");  // signed
   itoa_hex_long(adc_t);
   strcat(line,nstr);

   strcat(line,"    adc_h: ");  // unsigned
   itoa_hex_long(adc_h);
   strcat(line,nstr);

   strcat(line,"\r\n\n");
   uart_puts(UART_ID,line);  
}
//==============================================
void log_vars()
{
   unsigned int n;

   strcpy(line,"variables:   ");

   for(n=0; n<5; n++)
   {
      itoa_hex_long(testvars[n]);
      strcat(line,nstr);

      if(n == 7)  { strcat(line,"\n"); } 
      else { strcat(line,"   "); }
   }

   strcat(line,"\r\n\n");
   uart_puts(UART_ID,line);  
}
//==========================================================
// int64
uint32_t compensate_pressure()
{
    int64_t var1;
    int64_t var2;
    int64_t var3;
    int64_t var4;
    uint32_t pressure;
    uint32_t pressure_min = 3000000;
    uint32_t pressure_max = 11000000;

    var1 = ((int64_t)t_fine) - 128000;
    var2 = var1 * var1 * (int64_t)dig_P6;
    var2 = var2 + ((var1 * (int64_t)dig_P5) * 131072);
    var2 = var2 + (((int64_t)dig_P4) * 34359738368);
    var1 = ((var1 * var1 * (int64_t)dig_P3) / 256) + ((var1 * ((int64_t)dig_P2) * 4096));
    var3 = ((int64_t)1) * 140737488355328;
    var1 = (var3 + var1) * ((int64_t)dig_P1) / 8589934592;

    /* To avoid divide by zero exception */
    if (var1 != 0)
    {
        var4 = 1048576 - adc_p;    // not pressure;  // adc_p?
        var4 = (((var4 * INT64_C(2147483648)) - var2) * 3125) / var1;
        var1 = (((int64_t)dig_P9) * (var4 / 8192) * (var4 / 8192)) / 33554432;
        var2 = (((int64_t)dig_P8) * var4) / 524288;
        var4 = ((var4 + var1 + var2) / 256) + (((int64_t)dig_P7) * 16);
        pressure = (uint32_t)(((var4 / 2) * 100) / 128);

        if (pressure < pressure_min)
        {
            pressure = pressure_min;
        }
        else if (pressure > pressure_max)
        {
            pressure = pressure_max;
        }
    }
    else
    {
        pressure = pressure_min;
    }
 
    Pressure = pressure/100;
    return pressure;
}
//============================================
static int32_t compensate_temperature()
{
    int32_t var1;
    int32_t var2;
    int32_t temperature;
    int32_t lbin;
    int16_t sign;

    temperature = adc_t;

    var1 = (int32_t)((temperature / 8) - ((int32_t)dig_T1 * 2));

    var1 = (var1 * ((int32_t)dig_T2)) / 2048;
testvars[0] = var1;

    var2 = (int32_t)((temperature / 16) - ((int32_t)dig_T1));
 
    var2 = (((var2 * var2) / 4096) * ((int32_t)dig_T3)) / 16384;
testvars[1] = var2;

    t_fine = var1 + var2;
testvars[2] = t_fine;

    temperature = (t_fine * 5 + 128) / 256;
testvars[3] = temperature;

// Scale and convert to Fahrenheit
   if(temperature<0)  {sign = 1;}
   else     {sign = 0;}

   T = temperature/10;   // Example: 2508 -> 250 

  if(sign)  { lbin = (T * 18) - 3200;}
  else { lbin = (T * 18) + 3200;}    // Example:  2508 -> 7714

   Fdeg = lbin/10;  // Example: 2508 -> 771 = 0x0302

   temperature = Fdeg;    // signed int
testvars[4] = Fdeg;

//   log_vars();

   return Fdeg;
}
//==============================
static uint32_t compensate_humidity()
{
    int32_t var1;
    int32_t var2;
    int32_t var3;
    int32_t var4;
    int32_t var5;

// Humidity Compensation
// Bosch code
//  Returns humidity in %RH as unsigned 32 bit integer in Q22.10 format (22 integer and 10 fractional bits).

// Bosch Code from https://github.com/BoschSensortec/BME280_driver
    var1 = t_fine - (int32_t)76800;
    var2 = (int32_t)adc_h * 16384;           // adc_h is the uncompensated humidity
    var3 = (int32_t)dig_H4 * 1048576;
    var4 = (int32_t)dig_H5 * var1;
    var5 = ( ((var2 - var3) - var4) + (int32_t)16384 ) / 32768;
    var2 = (var1 * (int32_t)dig_H6 ) / 1024;
    var3 = ( (var1 * (int32_t)dig_H3) ) / 2048;
    var4 = ( (var2 * (var3 + (int32_t)32768)) / 1024) + (int32_t)2097152;
    var2 = ( (var4 * (int32_t)dig_H2 ) + 8192) / 16384;
    var3 = var5 * var2;
    var4 = ((var3 / 32768) * (var3 / 32768)) / 128;
    var5 = var3 - ((var4 * (int32_t)dig_H1 ) / 16);
    var5 = (var5 < 0 ? 0 : var5);
    var5 = (var5 > 419430400 ? 419430400 : var5);
    Humidity = ( (uint32_t)var5 / 4096);

    // mine
    Humidity3 = Humidity/100;
    Humidity = Humidity/1000;

    if (Humidity > 99)
    {
       Humidity = 99;
    }

   return Humidity;
}
//=========================================
void LED_break()
{
   while(1)  
   {
      gpio_put(LED1,1);
      sleep_ms(250); 
      gpio_put(LED1,0);
      sleep_ms(250); 
   }
}
//=====================================================
// log_wx_data_entry(&flash_entry)
void log_wx_data_entry(struct log *ptr)
{
   log_time();   // time stamp in line

   strcat(line,"Temperature: ");
   temperature_itoa_str(ptr->Ftemperature, nstr);      // remove leading zeros; end with NULL
   strcat(line,nstr);
   strcat(line," Degrees    ");

   strcat(line,"Pressure: ");
   pressure_ltoa_str(ptr->Pressure,nstr);        // remove leading zeros; end with NULL
   strcat(line,nstr);
   strcat(line," mb    ");

   strcat(line,"Humidity: ");
   humidity_btoa_str(ptr->Humidity, nstr);
   strcat(line,nstr);

   strcat(line," %\r\n\n");

   uart_puts(UART_ID,line);  
}
//=======================================================
// Convert Binary signed temperature to BCD; 4 digits
// temperature from bme280 is in Celsius 
void temperature_itoa_str(int32_t DegF, char *str)      // remove leading zeros; end with NULL
{
   unsigned char bcd0, bcd1, bcd2, bcd3;
   unsigned char char1, char2, char3, char4;

   int i;
   unsigned char hcarry;
   unsigned int bin, sign;

// 0x08DD = 2269.; 2269/10 = 226; 226 * 18 = 4084; 4084 + 3200 = 7284 = 72.8 deg F;  
// 0xF723 = -2269.; -2269/10 = -226; 226 * 18 = -4084; -4084 + 3200 = -884 = -8.8 deg F;  

   if(DegF < 0)
   {
      bin = -DegF;
      sign = 1; 
   }
   else
   { 
       bin = DegF;
       sign = 0;
   }

// Convert to BCD
   bcd0 = 0;
   bcd1 = 0;
   bcd2 = 0;
   bcd3 = 0;

   for(i=0; i<16; i++)
   {
//---------- BCD0
      bcd0 = bcd0<<1;

      if(bin & 0x8000)
      {bcd0 = bcd0 +1;}

      bin = bin<<1;

      if(bcd0 >9)
      {
         bcd0 = bcd0 - 10;
         hcarry = 1;
      }
     else {hcarry = 0; }

//---------- BCD1
      bcd1 = bcd1<<1;
      bcd1 = bcd1 + hcarry;

      if(bcd1 >9)
      {
         bcd1 = bcd1 - 10;
         hcarry = 1;
      }
      else {hcarry = 0;}

//---------- BCD2
      bcd2 = bcd2<<1;
      bcd2 = bcd2 + hcarry;

      if(bcd2 >9)
      {
         bcd2 = bcd2 - 10;
         hcarry = 1;
      }
      else {hcarry = 0;}

//---------- BCD3
      bcd3 = bcd3<<1;
      bcd3 = bcd3 + hcarry;

      if(bcd3 >9)
      {
          bcd3 = bcd3 - 10;
//          hcarry = 1;
      }
//      else {hcarry = 0;}

   }  // end of converting to bcd

// Convert to ascii
   char1 = bcd3 + 0x30;
   char2 = bcd2 + 0x30;
   char3 = bcd1 + 0x30;
   char4 = bcd0 + 0x30;

/*
// Make the string; do not remove leading zeros
   if(sign)  
   { 
      str[0] = '-';
      str[1] = char1;
      str[2] = char2;
      str[3] = char3;
      str[4] = '.';
      str[5] = char4;
      str[6] = 0;
   }
   else
   {
      str[0] = char1;
      str[1] = char2;
      str[2] = char3;
      str[3] = '.';
      str[4] = char4;
      str[5] = 0;
   }
*/

// Make the string; remove leading zeros
    if(sign)    // number is negative
    { 
       str[0] = '-';

       if(char1 != 0x30)
       { 
          str[1] = char1;
          str[2] = char2;
          str[3] = char3;
          str[4] = '.';
          str[5] = char4;
          str[6] = 0;
          for(i=0; i<7; i++)  {temperature_lcd[i] = str[i];}  // for displaying on lcd later
          return;
       }
     
       if(char2 != 0x30)
       { 
          str[1] = char2;
          str[2] = char3;
          str[3] = '.';
          str[4] = char4;
          str[5] = 0;
          for(i=0; i<6; i++)  {temperature_lcd[i] = str[i];}  // for displaying on lcd later
          return;
       }  

      // display at least two characters
      str[1] = char3;
      str[2] = '.';
      str[3] = char4;
      str[4] = 0;
      for(i=0; i<5; i++)  {temperature_lcd[i] = str[i];}   // for displaying on lcd later
    }  // end of sign is negative
    else   // sign is positive
    {
       if(char1 != 0x30)
       { 
          str[0] = char1;
          str[1] = char2;
          str[2] = char3;
          str[3] = '.';
          str[4] = char4;
          str[5] = 0;
          for(i=0; i<6; i++)  {temperature_lcd[i] = str[i];}  // for displaying on lcd later
          return;
       }
     
       if(char2 != 0x30)
       { 
          str[0] = char2;
          str[1] = char3;
          str[2] = '.';
          str[3] = char4;
          str[4] = 0;
          for(i=0; i<5; i++)  {temperature_lcd[i] = str[i];}  // for displaying on lcd later
          return;
       }  

      // display at least two characters
      str[0] = char3;
      str[1] = '.';
      str[2] = char4;
      str[3] = 0;
      for(i=0; i<4; i++)  {temperature_lcd[i] = str[i];}    // for displaying on lcd later
    }  // end of sign is positive

}
//==========================================================
// Convert hex pressure to BCD; 6 digits
void pressure_ltoa_str(uint32_t bin, char *str)      // remove leading zeros; end with NULL
{
   unsigned char bcd0, bcd1, bcd2, bcd3, bcd4, bcd5;
   unsigned char char1, char2, char3, char4, char5;

   int i;
   unsigned char hcarry;

// Adapted from 8-bit microcontroller code using a DAA instruction
   bcd0 = 0;
   bcd1 = 0;
   bcd2 = 0;
   bcd3 = 0;
   bcd4 = 0;
   bcd5 = 0;

   for(i=0; i<24; i++)
   {
//---------- BCD0
      bcd0 = bcd0<<1;

      if(bin & 0x800000)
      {bcd0 = bcd0 +1;}

      bin = bin<<1;

      if(bcd0 >9)
      {
         bcd0 = bcd0 - 10;
         hcarry = 1;
      }
     else {hcarry = 0; }

//---------- BCD1
      bcd1 = bcd1<<1;
      bcd1 = bcd1 + hcarry;

      if(bcd1 >9)
      {
         bcd1 = bcd1 - 10;
         hcarry = 1;
      }
      else {hcarry = 0;}

//---------- BCD2
      bcd2 = bcd2<<1;
      bcd2 = bcd2 + hcarry;

      if(bcd2 >9)
      {
         bcd2 = bcd2 - 10;
         hcarry = 1;
      }
      else {hcarry = 0;}

//---------- BCD3
      bcd3 = bcd3<<1;
      bcd3 = bcd3 + hcarry;

      if(bcd3 >9)
      {
          bcd3 = bcd3 - 10;
          hcarry = 1;
      }
      else {hcarry = 0;}

//---------- BCD4
      bcd4 = bcd4<<1;
      bcd4 = bcd4 + hcarry;

      if(bcd4 >9)
      {
         bcd4 = bcd4 - 10;
         hcarry = 1;
      }
      else {hcarry = 0;}

//---------- BCD5
      bcd5 = bcd5<<1;
      bcd5 = bcd5 + hcarry;

      if(bcd5 >9)
      {
         bcd5 = bcd5 - 10;
//         hcarry = 1;
      }
//      else {hcarry = 0;}

   }  // end of converting to bcd

// Convert to ascii
   char1 = bcd5 + 0x30;
   char2 = bcd4 + 0x30;
   char3 = bcd3 + 0x30;
   char4 = bcd2 + 0x30;
   char5 = bcd1 + 0x30;

/*
   str[0] = char1;
   str[1] = char2;
   str[2] = char3;
   str[3] = char4;
   str[4] = '.';
   str[5] = char5;
   str[6] = 0;
*/

// Make the string; remove leading zeros
   if(char1 != 0x30)
   { 
      str[0] = char1;
      str[1] = char2;
      str[2] = char3;
      str[3] = char4;
      str[4] = '.';
      str[5] = char5;
      str[6] = 0;      // NULL
      for(i=0; i<7; i++)  {pressure_lcd[i] = str[i];}  // for displaying on lcd
      return;
   }
   
   if(char2 != 0x30)
   { 
      str[0] = char2;
      str[1] = char3;
      str[2] = char4;
      str[3] = '.';
      str[4] = char5;
      str[5] = 0;    // NULL
      for(i=0; i<6; i++)  {pressure_lcd[i] = str[i];}  // for displaying on lcd
      return;
   }

   if(char3 != 0x30)
   { 
      str[0] = char3;
      str[1] = char4;
      str[2] = '.';
      str[3] = char5;
      str[4] = 0;     // NULL
      for(i=0; i<5; i++)  {pressure_lcd[i] = str[i];}  // for displaying on lcd
      return;
   }

   str[0] = char4;
   str[1] = '.';
   str[2] = char5;
   str[3] = 0;     // NULL
   for(i=0; i<4; i++)  {pressure_lcd[i] = str[i];}  // for displaying on lcd

   asm(" nop");    // for emulator
}
//=============================================
// Convert Binary signed humidity to BCD - 4 digits
void humidity_btoa_str(uint8_t bin, char *str)      // remove leading zeros; end with NULL
{
   unsigned char bcd0, bcd1;
   unsigned char char1, char2;

   int i;
   unsigned char hcarry;

   if(bin>99)  bin=99;   // maximum number

// Adapted from 8-bit microcontroller code using a DAA instruction
   bcd0 = 0;
   bcd1 = 0;

   for(i=0; i<8; i++)
   {
//---------- BCD0
      bcd0 = bcd0<<1;

      if(bin & 0x80)
      {bcd0 = bcd0 +1;}

      bin = bin<<1;

      if(bcd0 >9)
      {
         bcd0 = bcd0 - 10;
         hcarry = 1;
      }
     else {hcarry = 0; }

//---------- BCD1
      bcd1 = bcd1<<1;
      bcd1 = bcd1 + hcarry;

      if(bcd1 >9)
      {
         bcd1 = bcd1 - 10;
//         hcarry = 1;
      }
       else {hcarry = 0;}
   }  // end of converting to bcd

// Convert to ascii
   char1 = bcd1 + 0x30;
   char2 = bcd0 + 0x30;

// Make the string; remove leading zeros
   if(char1 != 0x30)
   { 
      str[0] = char1;
      str[1] = char2;
      str[2] = 0;
      for(i=0; i<3; i++)  {humidity_lcd[i] = str[i];}
   }
   else
   {
      str[0] = char2;
      str[1] = 0;
      for(i=0; i<2; i++)  {humidity_lcd[i] = str[i];}  // for displaying on lcd
   }  
}
//==================================================
// make_wx_entry(&flash_entry);
void make_wx_entry(struct log *ptr)
{
   uint16_t n;

   i2c_get_time();
  
   ptr->ts_number = ts_number;
   ts_number++;

   ptr->ts_date = time_buffer[DATE];
   ptr->ts_day = time_buffer[DAY];
   ptr->ts_month = time_buffer[MONTH];
   ptr->ts_year = time_buffer[YEAR];

   ptr->ts_hours = time_buffer[HOURS];
   ptr->ts_minutes = time_buffer[MINS];
   ptr->ts_seconds = time_buffer[SECS];

   ptr->Ftemperature = Fdeg;
   
   ptr->Pressure = Pressure;

   ptr->Humidity = Humidity;

   ptr->flag = 0;    // flag for non-blank

   for(n=0; n<10; n++)   { ptr->data[n] = 0xFF; }   
}
//================================
// show_entry(&flash_entry);
void show_entry(struct log *ptr)
{
   uint16_t m, n;

// ts_number is the log number starting from Reset
   my_itoa(ptr->ts_number,nstr);  // ends with NULL
   strcpy(line, nstr);
   strcat(line,"    ");   

// day name
   n = (ptr->ts_day)- 1;   // use it as an index
   for(m=0; m<3; m++)
   {
      nstr[m] = day_array[n][m];
   }
   nstr[m] = 0;
   strcat(line,nstr);
   strcat(line,"    ");   

   line_fill(ptr->ts_month, nstr, 0, 1);
   strcat(line,nstr);
   strcat(line,"/");

   line_fill(ptr->ts_date, nstr, 0, 1);
   strcat(line,nstr);
   strcat(line,"/");

   line_fill(ptr->ts_year, nstr, 0, 1);
   strcat(line,nstr);
   strcat(line,"    ");

   line_fill(ptr->ts_hours, nstr, 0, 1);
   strcat(line,nstr);
   strcat(line,":");

   line_fill(ptr->ts_minutes, nstr, 0, 1);
   strcat(line,nstr);
   strcat(line,":");

   line_fill(ptr->ts_seconds, nstr, 0, 1);
   strcat(line,nstr);
   strcat(line,"    ");

   strcat(line, "Temperature: ");
   temperature_itoa_str( ptr->Ftemperature, nstr);      // remove leading zeros; end with NULL
   strcat(line,nstr);
   strcat(line," F     ");
   
   strcat(line, "Pressure: ");
   pressure_ltoa_str(ptr->Pressure, nstr);      // remove leading zeros; end with NULL
   strcat(line,nstr);
   strcat(line," mb      ");

   strcat(line, "Humidity: ");
   humidity_btoa_str( ptr->Humidity, nstr);      // remove leading zeros; end with NULL
   strcat(line,nstr);
   strcat(line," %\n");

/*
   strcpy(line, "Data: ");
   for(n=0; n<10; n++)
   {
      my_hextoa2(ptr->data[n], nstr);
      strcat(line,nstr);   
      strcat(line,"  ");   
   }
*/
   strcat(line," \n");
   uart_puts(UART_ID, line); 
}
//=================================================

