// clock.h

#define DSADR  0x68   // addr is ds3231 address = DSADR = 110 1000 = 0x68

#define PB1 14
#define PB2 13
#define PB3 12
#define LED 2

// Messages - Duplicated in main
struct messages
{
    unsigned char pos;        // position
    unsigned char numchars;   // number of characters
    unsigned char msg[];      // message
};
//---------------------------------------------------------
// Declare the messages
const struct messages msg1 = {0x80, 8, "LCD Test"};
const struct messages msg2 = {0x80, 15, "Hr Min Sec Date"};

const struct messages msg3 = {0x85, 5, "Clock"};
const struct messages msg4 = {0x82, 13, "Set The Clock"};

const struct messages msg5 =  {0x80, 9, "Set Hours"};
const struct messages msg6 =  {0x80, 11, "Set Minutes"};
const struct messages msg7 =  {0x80, 7, "Set Day"};
const struct messages msg8 =  {0x80, 8, "Set Date"};
const struct messages msg9 =  {0x80, 9, "Set Month"};
const struct messages msg10 = {0x80, 8, "Set Year"};
const struct messages msg11 = {0x80, 14, "Set the Clock?"};
const struct messages msg12 = {0xC2, 8, "No   Yes"};
const struct messages msg13 = {0xC4, 4, "Done"};
const struct messages msg14 = {0x80, 15, "Ok, not changed"};
const struct messages msg15 = {0x80, 9, "Set DST"};

// Messages for the days of the week
const char day_array[7][3] = {"Sun","Mon","Tue","Wed","Thu","Fri","Sat"};
// Messages for the days of the week

const char month_array[18][3] =
{"Jan","Feb","Mar","Apr","May","Jun","Jul","Aug","Sep","aaa","bbb","ccc","ddd","eee","fff",
 "Oct","Nov","Dec"};  // This is the easy way to convert bcd month to decimal

const char *month_name[12] =
{"January","February","March","April","May","June","July","August","September",
 "October","November","December"};
//================================================================
// For reading, debouncing, and processing the switches
struct myswitch
{
    unsigned char debounce;     // debounce byte
    unsigned char state;        // debounced state of the switch
    unsigned char old;          // old state
    unsigned char press;        // switch has been pressed
    unsigned char count;        // how long the press counter has been running
    unsigned char release;      // how long the release counter has been running
    unsigned char countrun;     // is the counter running?
};

// Declare and initialize the switches
struct myswitch switch1 = {0xFF, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00};
struct myswitch switch2 = {0xFF, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00};
struct myswitch switch3 = {0xFF, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00};
struct myswitch switch4 = {0xFF, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00};

void do_switch(unsigned char, struct myswitch *);
//=======================================
// Clock State functions
void state0(void), state1(void), state2(void), state3(void), state4(void), state5(void),
     state6(void), state7(void), state8(void), state9(void), state10(void);

// State array so they can be called by an index
void (*state_ptr[11])() = {state0,state1,state2,state3,state4,state5,state6,state7,
                           state8, state9, state10};
//=======================================
extern void lcd_text(char *);
extern void SetCmdMode(void);
extern void SetChrMode(void);
extern void lcd_byte(char);  // clear screen
extern void domsg(const struct messages *);
extern void lcd_wr_instr(unsigned char);
extern void lcd_wr_data(unsigned char);

extern unsigned char irq_flag, timer_flag, nflag;
extern unsigned int save_count; 
extern uint8_t out_buffer[10];
extern unsigned int irq_count;

uint8_t output[1];   // register address start?
uint8_t time_buffer[7];
unsigned int state;

#define SECS   0
#define MINS   1
#define HOURS  2
#define DAY    3
#define DATE   4
#define MONTH  5
#define YEAR   6
//======================================================
unsigned char inc_bcd(unsigned char, unsigned char, unsigned char);
unsigned char dec_bcd(unsigned char, unsigned char, unsigned char);
extern void lcd_wr_time(unsigned int, unsigned int);
extern void clear_lcd(unsigned char);

void i2c_get_time(void);
void do_clock(void);

extern void LED_break(void);
//======================================================
