// lcd.c  lcd functions
#include <stdio.h>
#include <string.h>
#include "pico/stdlib.h"
#include "hardware/i2c.h"
#include "pico/binary_info.h"

#define LCD_E 10
#define LCD_RS 11
#define LCD_D4 6
#define LCD_D5 7
#define LCD_D6 8
#define LCD_D7 9

struct messages
{
    unsigned char pos;        // position
    unsigned char numchars;   // number of characters
    unsigned char msg[];      // message
};

void SetChrMode(void);
void SetCmdMode(void);
void lcd_init(void);
void domsg(const struct messages *);
void lcd_wr_data(unsigned char);
void lcd_wr_instr(unsigned char);
void lcd_text(char *);
void lcd_byte(char);
void lcd_wr_hex(unsigned int, unsigned char);
void lcd_wr_hex16(unsigned int, unsigned int);
void convert_hex_bcd2(unsigned char, unsigned char *);
void lcd_wr_bcd_A(unsigned int, unsigned int);
void lcd_wr_bcd2(unsigned int, unsigned int);
void lcd_wr_time(unsigned int, unsigned int);
void clear_lcd(unsigned char);
//================================================================
// First Convert Binary to ASCII
// Adapted from 8-bit microcontroller code using a DAA instruction
void lcd_wr_time(unsigned int pos, unsigned int bcd)
{
   unsigned char char1, char2;

// Convert to ascii
   char1 = (bcd>>4) + 0x30;
   char2 = (bcd & 0x0F) + 0x30;

   lcd_wr_instr(pos);
   lcd_wr_data(char1);
   lcd_wr_data(char2);
}
//====================================
// Convert unsigned hex byte to BCD and display it 
void lcd_wr_bcd(unsigned int pos, unsigned char bin)
{
// First Convert Binary to BCD
// Adapted from 8-bit microcontroller code with a DAA instruction

   unsigned char bcd0 = 0;
   unsigned char bcd1 = 0;
   unsigned char bcd2 = 0;
   unsigned char char1, char2, char3;
   int i;
   unsigned char hcarry;

   for(i=0; i<8; i++)
   {
//---------- BCD0
      bcd0 = bcd0<<1;

      if(bin & 0x80)
      {bcd0 = bcd0 +1;}

      bin = bin<<1;

      if(bcd0 >9)
      {
         bcd0 = bcd0 - 10;
         hcarry = 1;
      }
      else {hcarry = 0; }

//---------- BCD1
      bcd1 = bcd1<<1;
      bcd1 = bcd1 + hcarry;

      if(bcd1 >9)
      {
          bcd1 = bcd1 - 10;
          hcarry = 1;
      }
      else {hcarry = 0;}

//---------- BCD2
      bcd2 = bcd2<<1;
      bcd2 = bcd2 + hcarry;

     if(bcd2 >9)
     {
         bcd2 = bcd2 - 10;
         hcarry = 1;
     }
         else {hcarry = 0;}
   }

// Convert to ascii
   char1 = bcd2 + 0x30;
   char2 = bcd1 + 0x30;
   char3 = bcd0 + 0x30;

// Display it
   lcd_wr_instr(pos);

   if(char1 == 0x30)
    { lcd_wr_data(' '); }
   else
    { lcd_wr_data(char1);}

   if( (char1 == 0x30) & (char2 == 0x30) )
    { lcd_wr_data(' '); }
   else
    { lcd_wr_data(char2);}

   lcd_wr_data(char3);
}
//====================================
// Convert unsigned hex byte to BCD and display it 
void lcd_wr_bcd2(unsigned int pos, unsigned int bin)
{
// First Convert Binary to BCD
// Adapted from 8-bit microcontroller code with a DAA instruction

   unsigned char bcd0 = 0;
   unsigned char bcd1 = 0;
   unsigned char char1, char2;
   int i;
   unsigned char hcarry;

   for(i=0; i<8; i++)
   {
//---------- BCD0
      bcd0 = bcd0<<1;

      if(bin & 0x80)
      {bcd0 = bcd0 +1;}

      bin = bin<<1;

      if(bcd0 >9)
      {
         bcd0 = bcd0 - 10;
         hcarry = 1;
      }
      else {hcarry = 0; }

//---------- BCD1
      bcd1 = bcd1<<1;
      bcd1 = bcd1 + hcarry;

      if(bcd1 >9)
      {
          bcd1 = bcd1 - 10;
//          hcarry = 1;
      }
      else {hcarry = 0;}


// Convert to ascii
   char1 = bcd1 + 0x30;
   char2 = bcd0 + 0x30;
 
// Display it
   lcd_wr_instr(pos);
   lcd_wr_data(char1);
   lcd_wr_data(char2);}
}
//====================================
void convert_hex_bcd2(unsigned char bin, unsigned char *outstring)
{
// Adapted from 8-bit microcontroller code with a DAA instruction

   unsigned char bcd0 = 0;
   unsigned char bcd1 = 0;
   int i;
   unsigned char hcarry;

   for(i=0; i<8; i++)
   {
//---------- BCD0
      bcd0 = bcd0<<1;

      if(bin & 0x80)
      {bcd0 = bcd0 +1;}

      bin = bin<<1;

      if(bcd0 >9)
      {
         bcd0 = bcd0 - 10;
         hcarry = 1;
      }
      else {hcarry = 0; }

//---------- BCD1
      bcd1 = bcd1<<1;
      bcd1 = bcd1 + hcarry;

      if(bcd1 >9)
      {
         bcd1 = bcd1 - 10;
         hcarry = 1;
      }
      else {hcarry = 0;}
   }

// Convert to ascii
   outstring[0] = bcd1 + 0x30;
   outstring[1] = bcd0 + 0x30;
}
//====================================
void lcd_wr_hex(unsigned int pos, unsigned char bin)
{
   unsigned char1, char2, temp;

// Convert to ascii
   temp = bin & 0x0F;
   if(temp<10)
     {char2 = temp + 0x30;}
   else
     {char2 = temp + 0x37;}

   temp = bin>>4 & 0x0F;
   if(temp<10)
     {char1 = temp + 0x30;}
   else
     {char1 = temp + 0x37;}
    
// Display it
   lcd_wr_instr(pos);
   lcd_wr_data(char1);
   lcd_wr_data(char2);
}
//==========================================================
void lcd_wr_hex16(unsigned int pos, unsigned int bin)
{
   unsigned char1, char2, char3, char4, temp;

// Convert to ascii
   temp = bin & 0x0F;
   if(temp<10)
     {char4 = temp + 0x30;}
   else
     {char4 = temp + 0x37;}

   temp = bin>>4 & 0x0F;
   if(temp<10)
     {char3 = temp + 0x30;}
   else
     {char3 = temp + 0x37;}

   temp = bin>>8 & 0x0F;
   if(temp<10)
     {char2 = temp + 0x30;}
   else
     {char2 = temp + 0x37;}

   temp = bin>>12 & 0x0F;
   if(temp<10)
     {char1 = temp + 0x30;}
   else
     {char1 = temp + 0x37;}

// Display it
   lcd_wr_instr(pos);
   lcd_wr_data(char1);
   lcd_wr_data(char2);
   lcd_wr_data(char3);
   lcd_wr_data(char4);
}
//===============================================
void pulseEnable ()
{
   gpio_put(LCD_E,1);
   sleep_us(800);   // 800 microseconds

   gpio_put(LCD_E,0);
   sleep_us(800);
}
//==========================================
//  send a byte to the lcd in two nibbles
//  before calling use SetChrMode or SetCmdMode to determine whether to send character or command
void lcd_byte(char bits)
{
   if(bits & 0x10)  {gpio_put(LCD_D4,1);}
   else {gpio_put(LCD_D4,0);}

   if(bits & 0x20)  {gpio_put(LCD_D5,1);}
   else {gpio_put(LCD_D5,0);}

   if(bits & 0x40)  {gpio_put(LCD_D6,1);}
   else {gpio_put(LCD_D6,0);}

   if(bits & 0x80)  {gpio_put(LCD_D7,1);}
   else {gpio_put(LCD_D7,0);}

   pulseEnable();

   if(bits & 0x01)  {gpio_put(LCD_D4,1);}
   else {gpio_put(LCD_D4,0);}

   if(bits & 0x02)  {gpio_put(LCD_D5,1);}
   else {gpio_put(LCD_D5,0);}

   if(bits & 0x04)  {gpio_put(LCD_D6,1);}
   else {gpio_put(LCD_D6,0);}

   if(bits & 0x08)  {gpio_put(LCD_D7,1);}
   else {gpio_put(LCD_D7,0);}

   pulseEnable();      	
}
//--------------------------------------------
void SetCmdMode() 
{
  gpio_put(LCD_RS,0);   // command mode
}
//--------------------------------------------
void SetChrMode() 
{
   gpio_put(LCD_RS,1);  // data mode
}
//-------------------------------------------
void lcd_text(char *s)
{
  while(*s) 
	lcd_byte(*s++);
 }
//--------------------------------------------
void lcd_init()
{
// set up pi pins for output
    gpio_init(LCD_E);
    gpio_set_dir(LCD_E, GPIO_OUT);
    gpio_put(LCD_E, 0);

    gpio_init(LCD_RS);
    gpio_set_dir(LCD_RS, GPIO_OUT);
    gpio_put(LCD_RS, 0);

    gpio_init(LCD_D4);
    gpio_set_dir(LCD_D4, GPIO_OUT);
    gpio_put(LCD_D4, 0);

    gpio_init(LCD_D5);
    gpio_set_dir(LCD_D5, GPIO_OUT);
    gpio_put(LCD_D5, 0);

    gpio_init(LCD_D6);
    gpio_set_dir(LCD_D6, GPIO_OUT);
    gpio_put(LCD_D6, 0);

    gpio_init(LCD_D7);
    gpio_set_dir(LCD_D7, GPIO_OUT);
    gpio_put(LCD_D7, 0);

   // initialise LCD
   SetCmdMode(); // set for commands
   lcd_byte(0x33); // full init 
   lcd_byte(0x32); // 4 bit mode
   lcd_byte(0x28); // 2 line mode
   lcd_byte(0x0C); // display on, cursor off, blink off
   lcd_byte(0x01);  // clear screen

   sleep_ms(3);
}
//=================================================
// Do LCD Message
void domsg(const struct messages *sptr)
{
    unsigned char m,n;
    unsigned char data;

    m = sptr->pos;  // position
    lcd_wr_instr(m);

    m = sptr->numchars;  // numbers of chars

    for(n=0; n<m; n++)
    {
        data = sptr->msg[n];
        lcd_wr_data(data);
    }
}
//==========================================
// LCD - Write 8-bit instruction
// LCD RS (Register Select; RS low = Instruction, RS High = Data)
void lcd_wr_instr(unsigned char data)
{
//   sleep_us(60);
   gpio_put(LCD_RS,0);   // RS = 0; command mode
//   sleep_us(60);
   lcd_byte(data);
   gpio_put(LCD_RS,1);   // RS = 0; data mode
//   sleep_us(60);
}
//======================
// LCD - Write 8-bit data
// LCD RS (Register Select; RS low = Instruction, RS High = Data)
void lcd_wr_data(unsigned char data)
{
    gpio_put(LCD_RS,1);   // data mode, RS = 1
    lcd_byte(data);
}
//=================================================
void lcd_wr_bcd_A(unsigned int pos, unsigned int bin)
{
// First Convert Binary to ASCII
// Adapted from 8-bit microcontroller code with a DAA instruction

// bin = 1234; // 0x67 = 103d; 0x386 = 902d
   unsigned char bcd0 = 0;
   unsigned char bcd1 = 0;
   unsigned char bcd2 = 0;
   unsigned char bcd3 = 0;
   unsigned char char1, char2, char3, char4;
   int i;
   unsigned char hcarry;

   for(i=0; i<16; i++)
   {
//---------- BCD0
      bcd0 = bcd0<<1;

      if(bin & 0x8000)
      {bcd0 = bcd0 +1;}

      bin = bin<<1;

      if(bcd0 >9)
      {
         bcd0 = bcd0 - 10;
         hcarry = 1;
      }
      else {hcarry = 0; }

//---------- BCD1
      bcd1 = bcd1<<1;
      bcd1 = bcd1 + hcarry;

      if(bcd1 >9)
      {
          bcd1 = bcd1 - 10;
          hcarry = 1;
      }
      else {hcarry = 0;}

//---------- BCD2
      bcd2 = bcd2<<1;
      bcd2 = bcd2 + hcarry;

     if(bcd2 >9)
     {
         bcd2 = bcd2 - 10;
         hcarry = 1;
     }
         else {hcarry = 0;}

//---------- BCD3
     bcd3 = bcd3<<1;
     bcd3 = bcd3 + hcarry;

     if(bcd3 >9)
     {
         bcd3 = bcd3 - 10;
//       hcarry = 1;
     }
         else {hcarry = 0;}

   }

// Convert to ascii
   char1 = bcd3 + 0x30;
   char2 = bcd2 + 0x30;
   char3 = bcd1 + 0x30;
   char4 = bcd0 + 0x30;

// Display it
   lcd_wr_instr(pos);

   if(char1 == 0x30)
    { lcd_wr_data(' '); }
   else
    { lcd_wr_data(char1);}

   if(char2 == 0x30)
    { lcd_wr_data(' '); }
   else
    { lcd_wr_data(char2);}

   lcd_wr_data(char3);
   lcd_wr_data('.');

   lcd_wr_data(char4);
   lcd_wr_data(' ');
   lcd_wr_data('A');

//   lcd_wr_data('z');

}
//==========================================================
void clear_lcd(unsigned char pos)
{
    unsigned int n;

    lcd_wr_instr(pos);

    for(n=16; n>0; --n)
    { lcd_wr_data(' ');}
}
//=========================================================
