// main.h
//=======================================================================
const char version[64] = "\nJed Margolin's BME-280 Log   7/10/2024";  // for uart

// GPIO Pin Assigments
// LCD
#define LCD_E 10
#define LCD_RS 11
#define LCD_D4 6
#define LCD_D5 7
#define LCD_D6 8
#define LCD_D7 9

// Switches
#define PB1 14
#define PB2 13
#define PB3 12
#define SW 15    // slide switch

// RF Channel Select (on Header)
#define CHANNEL 1    

// UART (on Header)
#define UART_TX_PIN 0
#define UART_RX_PIN 1
#define UART_ID uart0
#define BAUD_RATE 115200

#define LED1 2    // Green
#define LED2 27   // Red

// nrf905 RF Module
#define TXEN 21    // nrf905 Transmit Enable
#define TRX_CE 20  // enable nrf905 chip for transmit/receive
#define AM 28      // Address Match
#define DR 22      // Data Ready
#define CSN905 17  // nrf906 CSN - pin 22
#define BUF_LEN  33

#define ADC0_PIN 26
//=======================================================================
// Functions for Pusbutton Switches
extern struct myswitch switch1;
extern struct myswitch switch2;
extern struct myswitch switch3;
extern void do_switch(unsigned char, struct myswitch *); 

extern void do_clock(void);
extern void do_bme280(void);

extern unsigned char timer_flag;
extern unsigned char time_buffer[];

#define SECS   0
#define MINS   1
#define HOURS  2
#define DAY    3
#define DATE   4
#define MONTH  5
#define YEAR   6

#define BME_ID 0x76  // BMP280 I2C device ID
//================================================
// For nrf905
// WC1[0] is the instruction and the address start for nrf905

// 434.0 MHz
unsigned char WC1[11] = { 0x00, 0x74, 0x0C, 0x44, 0x20, 0x20, 0x34, 0x33, 0x32, 0x31, 0xDB };  // 434.0 MHz
unsigned char TXADR1[5] = { 0x22, 0x34, 0x33, 0x32, 0x31 };   // TXADR[0] is the instruction

// 433.6 MHz
unsigned char WC2[11] = { 0x00, 0x70, 0x0C, 0x44, 0x20, 0x20, 0x71, 0x72, 0x73, 0x74, 0xDB };  // 433.6 MHz
unsigned char TXADR2[5] = { 0x22, 0x71, 0x72, 0x73, 0x74 };   // TXADR[0] is the instruction

// 433.4 MHz
unsigned char WC3[11] = { 0x00, 0x6E, 0x0C, 0x44, 0x20, 0x20, 0x52, 0x53, 0x54, 0x55, 0xDB };  // 433.4 MHz
unsigned char TXADR3[5] = { 0x22, 0x52, 0x53, 0x54, 0x55 };   // TXADR[0] is the instruction

unsigned char spi_out_buffer[BUF_LEN];   // out_buffer[0] is the instruction
unsigned char spi_in_buffer[BUF_LEN];    // in_buffer[0] is the status byte

void nrf905_init(void);
void nrf905_tx(void);
void nrf905_rx(void);
void nrf905_rx_test(void);

// Messages (duplicate in clock.h)
struct messages
{
    unsigned char pos;        // position
    unsigned char numchars;   // number of characters
    unsigned char msg[];      // message
};

// LCD Functions
extern void SetChrMode(void);
extern void SetCmdMode(void);
extern void lcd_init(void);
extern void domsg(const struct messages *);
extern void lcd_wr_data(unsigned char);
extern void lcd_wr_instr(unsigned char);
extern void lcd_text(char *);
extern void lcd_byte(char);
extern void lcd_wr_hex(unsigned int, unsigned char);
extern void convert_hex_bcd2(unsigned char, unsigned char *);
extern void lcd_wr_bcd(unsigned int, unsigned char);
extern void lcd_wr_bcd2(unsigned int, unsigned char);
extern void lcd_wr_time(unsigned int, unsigned int);
extern void lcd_wr_hex16(unsigned int, unsigned int);
extern void clear_lcd(unsigned char);

extern unsigned char i2c_outbuffer[10];
extern unsigned char i2c_inbuffer[128];

uint8_t out_buffer[10];     // first byte is register address

struct repeating_timer timer;
//=======================================================
void do_led_break(void);
void blink_led(unsigned int, unsigned int);

const struct messages dmsg1 = {0x80, 14,"Jed Margolin's"};
const struct messages dmsg2 = {0xC0, 11, "BME-280 Log"};
const struct messages dmsg3 = {0x81, 8, "7/10/2024"};
const struct messages dmsg4 = {0x80, 9, "Device ID"};
const struct messages dmsg5 = {0xC0, 7, "BME-280"};
const struct messages dmsg6 = {0xC0, 7, "BMP-280"};
const struct messages dmsg7 = {0xC0, 7, "Unknown"};
//=================================================================
unsigned int mode;   // 0 = bme-280; 1 = clock
unsigned int secs_flag, irq_flag;
unsigned int irq_count, save_count; 
unsigned char nflag, led_flag;
unsigned char device_id;  //save for later
//================================================================
extern char line[512];
void LED_break(void);
unsigned char uart_char;

// Find the first blank Page_Index (and Sector_index)
extern void find_first_blank_page(void);  

// If the SDATE page is blank, write today as the Start Date, otherwise show the Start Date
extern void do_start_date(uint16_t);  // Show Start Date
extern char start_date[20];
